@@ -119,7 +119,7 @@ void DemuxerThr::loadImage()
                     }
                 }
             }
-            if (img.isNull() && !artist.isEmpty() && (!title.isEmpty() || !album.isEmpty())) //Ładowanie okładki z cache
+            if (img.isNull() && (!artist.isEmpty() || title.contains(QStringLiteral(" - "))) && (!title.isEmpty() || !album.isEmpty())) //Ładowanie okładki z cache
             {
                 QString coverPath = getCoverFile(title, artist, album);
                 if (!title.isEmpty() && !album.isEmpty() && !QFile::exists(coverPath)) //Try to load cover for title if album cover doesn't exist
@@ -287,7 +287,7 @@ void DemuxerThr::seek(bool doDemuxerSeek)
             }
             else
             {
-                emit playC.chText(tr("Playback"));
+                changeStatusText();
             }
         }
     }
@@ -456,7 +456,7 @@ void DemuxerThr::run()
         updateBufferedSeconds = false;
 
     emit playC.updateLength(round(demuxer->length()));
-    emit playC.chText(tr("Playback"));
+    changeStatusText();
     emit playC.playStateChanged(true);
 
     localStream = demuxer->localStream();
@@ -534,6 +534,13 @@ void DemuxerThr::run()
     if (unknownLength)
         emit allowRecording(true);
 
+    auto handleRecording = [&](bool startAndStop) {
+        if (startAndStop && m_recording && !m_recMuxer)
+            startRecordingInternal(recStreamsMap);
+        else if (!m_recording && m_recMuxer)
+            stopRecordingInternal(recStreamsMap);
+    };
+
     while (!demuxer.isAborted())
     {
         {
@@ -543,10 +550,7 @@ void DemuxerThr::run()
                 break; //Repeat seek failed - break.
         }
 
-        if (m_recording && !m_recMuxer)
-            startRecordingInternal(recStreamsMap);
-        else if (!m_recording && m_recMuxer)
-            stopRecordingInternal(recStreamsMap);
+        handleRecording(true);
 
         AVThread *aThr = (AVThread *)playC.aThr, *vThr = (AVThread *)playC.vThr;
 
@@ -675,7 +679,10 @@ void DemuxerThr::run()
                 if (playC.seekTo == SEEK_STREAM_RELOAD)
                     break;
                 if (!first)
+                {
                     msleep(15);
+                    handleRecording(false);
+                }
                 else
                 {
                     msleep(1);
@@ -799,6 +806,7 @@ void DemuxerThr::startRecordingInternal(QHash<int, int> &recStreamsMap)
         recStreamsMap.clear();
         emit recording(false, true);
     }
+    changeStatusText();
 }
 void DemuxerThr::stopRecordingInternal(QHash<int, int> &recStreamsMap)
 {
@@ -807,6 +815,7 @@ void DemuxerThr::stopRecordingInternal(QHash<int, int> &recStreamsMap)
     m_recMuxer.reset();
     recStreamsMap.clear();
     emit recording(false, false);
+    changeStatusText();
 }
 
 inline void DemuxerThr::ensureTrueUpdateBuffered()
@@ -824,7 +833,7 @@ void DemuxerThr::handlePause()
         if (!paused)
         {
             paused = true;
-            emit playC.chText(tr("Paused"));
+            changeStatusText();
             emit playC.playStateChanged(false);
             playC.emptyBufferCond.wakeAll();
         }
@@ -832,7 +841,7 @@ void DemuxerThr::handlePause()
     else if (paused)
     {
         paused = demuxerPaused = false;
-        emit playC.chText(tr("Playback"));
+        changeStatusText();
         emit playC.playStateChanged(true);
         playC.emptyBufferCond.wakeAll();
     }
@@ -1406,6 +1415,18 @@ double DemuxerThr::getFrameDelay() const
     return 1.0 / demuxer->streamsInfo().at(playC.videoStream)->getFPS();
 }
 
+void DemuxerThr::changeStatusText()
+{
+    if (paused)
+    {
+        emit playC.chText(tr("Paused"));
+    }
+    else
+    {
+        emit playC.chText(m_recording ? tr("Playback and recording") : tr("Playback"));
+    }
+}
+
 void DemuxerThr::stopVADec()
 {
     clearBuffers();
@@ -1445,7 +1466,15 @@ void DemuxerThr::updateCover(const QString &title, const QString &artist, const
     const QImage coverImg = QImage::fromData(cover);
     if (!coverImg.isNull())
     {
-        if (this->title == title && this->artist == artist && (this->album == album || (album.isEmpty() && !title.isEmpty())))
+        const bool bothInTitle =
+               !title.isEmpty()
+            && !artist.isEmpty()
+            && this->artist.isEmpty()
+            && this->title.contains(QStringLiteral(" - "))
+            && this->title.contains(title)
+            && this->title.contains(artist)
+        ;
+        if ((bothInTitle || (this->title == title && this->artist == artist)) && (this->album == album || (album.isEmpty() && !title.isEmpty())))
             emit playC.updateImage(coverImg);
 
         static bool useCoversCache = !QMPlay2Core.getSettings().getBool("NoCoversCache");
@@ -1454,7 +1483,7 @@ void DemuxerThr::updateCover(const QString &title, const QString &artist, const
 
         QDir dir(QMPlay2Core.getSettingsDir());
         dir.mkdir("Covers");
-        QFile f(getCoverFile(title, artist, album));
+        QFile f(bothInTitle ? getCoverFile(this->title, QString(), album) : getCoverFile(title, artist, album));
         if (f.open(QFile::WriteOnly))
         {
             f.write(cover);
