@@ -18,7 +18,7 @@
 
 #pragma once
 
-#include <QWidget>
+#include <QScrollArea>
 
 class QRadioButton;
 
@@ -26,10 +26,10 @@ namespace Ui {
     class OSDSettings;
 }
 
-class OSDSettingsW : public QWidget
+class OSDSettingsW : public QScrollArea
 {
 public:
-    static void init(const QString &, int, int, int, int, int, int, double, double, const QColor &, const QColor &, const QColor &);
+    static void init(const QString &, int, int, int, int, int, int, double, double, const QColor &, const QColor &, const QColor &, bool l);
 
     OSDSettingsW(const QString &);
     ~OSDSettingsW();
