@@ -36,6 +36,15 @@
 #include <QMessageBox>
 #include <QRandomGenerator>
 
+static bool urlMatchesWithItem(QTreeWidgetItem *item, const QString &url)
+{
+    QString itemUrl = item->data(0, Qt::UserRole).toString();
+    bool urlMatches = (itemUrl == url);
+    if (!urlMatches && Functions::splitPrefixAndUrlIfHasPluginPrefix(itemUrl, nullptr, &itemUrl, nullptr))
+        urlMatches = (itemUrl == url); //E.g. for chiptunes without group
+    return urlMatches;
+}
+
 PlaylistDock::PlaylistDock() :
     repeatMode(RepeatNormal),
     lastPlaying(nullptr)
@@ -124,6 +133,7 @@ bool PlaylistDock::save(const QString &_url, bool saveCurrentGroup)
 
 void PlaylistDock::add(const QStringList &urls)
 {
+    list->dontUpdateAfterAdd = false;
     list->add(urls);
 }
 void PlaylistDock::addAndPlay(const QStringList &urls)
@@ -146,11 +156,7 @@ void PlaylistDock::addAndPlay(const QString &_url)
     const QString url = Functions::Url(_url);
     for (QTreeWidgetItem *item : items)
     {
-        QString itemUrl = item->data(0, Qt::UserRole).toString();
-        bool urlMatches = (itemUrl == url);
-        if (!urlMatches && Functions::splitPrefixAndUrlIfHasPluginPrefix(itemUrl, nullptr, &itemUrl, nullptr))
-            urlMatches = (itemUrl == url); //E.g. for chiptunes without group
-        if (urlMatches)
+        if (urlMatchesWithItem(item, url))
         {
             playAfterAdd = true;
             if (list->isGroup(item))
@@ -169,6 +175,36 @@ void PlaylistDock::addAndPlay(const QString &_url)
     addAndPlay(QStringList{_url});
 }
 
+void PlaylistDock::remove(const QString &url)
+{
+    if (!url.isEmpty())
+        remove(QStringList{url});
+}
+void PlaylistDock::remove(const QStringList &urls)
+{
+    if (!list->canModify())
+        return;
+    QTreeWidgetItem *par = initializeItemsDelete();
+    bool deleted = false;
+    auto items = list->getChildren(PlaylistWidget::ALL_CHILDREN);
+    for (auto &&url : urls)
+    {
+        for (auto it = items.begin(); it != items.end();)
+        {
+            if (urlMatchesWithItem(*it, url) && maybeDeleteTreeWidgetItem(*it))
+            {
+                it = items.erase(it);
+                deleted = true;
+            }
+            else
+            {
+                ++it;
+            }
+        }
+    }
+    finalizeItemsDelete(par, deleted);
+}
+
 void PlaylistDock::scrollToCurrectItem()
 {
     list->scrollToItem(list->currentItem());
@@ -240,12 +276,43 @@ void PlaylistDock::doGroupSync(bool quick, QTreeWidgetItem *tWI, bool quickRecur
     }
 }
 
-void PlaylistDock::deleteTreeWidgetItem(QTreeWidgetItem *tWI)
+bool PlaylistDock::maybeDeleteTreeWidgetItem(QTreeWidgetItem *tWI)
 {
+    if (PlaylistWidget::getFlags(tWI) & Playlist::Entry::Locked)
+        return false;
     randomPlayedItems.removeOne(tWI);
     if (lastPlaying == tWI)
         lastPlaying = nullptr;
     delete tWI;
+    return true;
+}
+
+QTreeWidgetItem *PlaylistDock::initializeItemsDelete()
+{
+    QTreeWidgetItem *par = nullptr;
+    if (auto currentItem = list->currentItem())
+        par = currentItem->parent();
+    list->setItemsResizeToContents(false);
+    return par;
+}
+void PlaylistDock::finalizeItemsDelete(QTreeWidgetItem *par, bool deleted)
+{
+    if (deleted)
+    {
+        list->refresh();
+        if (auto currentItem = list->currentItem())
+            par = currentItem;
+        else if (!list->getChildren().contains(par))
+            par = nullptr;
+        if (!par)
+            par = list->topLevelItem(0);
+        list->setCurrentItem(par);
+    }
+    list->setItemsResizeToContents(true);
+    if (deleted)
+    {
+        list->processItems();
+    }
 }
 
 void PlaylistDock::itemDoubleClicked(QTreeWidgetItem *tWI)
@@ -345,14 +412,11 @@ void PlaylistDock::next(bool playingError)
         }
         else
         {
-            int entryFlags = PlaylistWidget::getFlags(lastPlaying);
+            const int entryFlags = PlaylistWidget::getFlags(lastPlaying);
             if (entryFlags & Playlist::Entry::StopAfter)
             {
-                //Remove "StopAfter" flag and stop playback.
-                entryFlags &= ~Playlist::Entry::StopAfter;
-                PlaylistWidget::setEntryFont(lastPlaying, entryFlags);
-                lastPlaying->setData(0, Qt::UserRole + 1, entryFlags);
                 //"tWI" is nullptr here, so playback will stop.
+                Q_ASSERT(!tWI);
             }
             else
             {
@@ -491,31 +555,51 @@ void PlaylistDock::newGroup()
     list->setCurrentItem(list->newGroup());
     entryProperties();
 }
-
-void PlaylistDock::delEntries()
+void PlaylistDock::delEntries(bool fromDisk)
 {
     if (!isVisible() || !list->canModify()) //jeżeli jest np. drag and drop to nie wolno usuwać
         return;
     const QList<QTreeWidgetItem *> selectedItems = list->selectedItems();
-    if (!selectedItems.isEmpty())
+    if (selectedItems.isEmpty())
+        return;
+
+    if (fromDisk &&
+            QMessageBox::question(
+                this,
+                tr("Playlist"),
+                tr("Are you sure you want to delete selected entries from disk? "
+                   "Non-local entries will only be removed from the list."),
+                QMessageBox::Yes,
+                QMessageBox::No) == QMessageBox::No) {
+        return;
+    }
+
+    QTreeWidgetItem *par = initializeItemsDelete();
+    bool deleted = false;
+    bool diskDeleteFailed = false;
+    for (QTreeWidgetItem *tWI : selectedItems)
     {
-        QTreeWidgetItem *par = list->currentItem() ? list->currentItem()->parent() : nullptr;
-        list->setItemsResizeToContents(false);
-        for (QTreeWidgetItem *tWI : selectedItems)
+        bool canDeleteTWI = true;
+        if (fromDisk && !(PlaylistWidget::getFlags(tWI) & Playlist::Entry::Locked))
         {
-            if (!(PlaylistWidget::getFlags(tWI) & Playlist::Entry::Locked))
-                deleteTreeWidgetItem(tWI);
+            if (const QString url = getUrl(tWI); url.startsWith("file://"))
+            {
+                canDeleteTWI = QFile::moveToTrash(url.mid(7));
+                if (!canDeleteTWI)
+                    diskDeleteFailed = true;
+            }
         }
-        list->refresh();
-        if (list->currentItem())
-            par = list->currentItem();
-        else if (!list->getChildren().contains(par))
-            par = nullptr;
-        if (!par)
-            par = list->topLevelItem(0);
-        list->setCurrentItem(par);
-        list->setItemsResizeToContents(true);
-        list->processItems();
+        if (canDeleteTWI && maybeDeleteTreeWidgetItem(tWI))
+        {
+            deleted = true;
+        }
+    }
+    finalizeItemsDelete(par, deleted);
+
+    if (diskDeleteFailed)
+    {
+        Q_ASSERT(fromDisk);
+        QMessageBox::warning(this, tr("Playlist"), tr("Failed to delete one or more selected entries."));
     }
 }
 void PlaylistDock::delNonGroupEntries(bool force)
@@ -524,15 +608,14 @@ void PlaylistDock::delNonGroupEntries(bool force)
         return;
     if (force || QMessageBox::question(this, tr("Playlist"), tr("Are you sure you want to delete ungrouped entries?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
     {
-        list->setItemsResizeToContents(false);
+        QTreeWidgetItem *par = initializeItemsDelete();
+        bool deleted = false;
         for (QTreeWidgetItem *tWI : list->topLevelNonGroupsItems())
         {
-            if (!(PlaylistWidget::getFlags(tWI) & Playlist::Entry::Locked))
-                deleteTreeWidgetItem(tWI);
+            if (maybeDeleteTreeWidgetItem(tWI))
+                deleted = true;
         }
-        list->refresh();
-        list->setItemsResizeToContents(true);
-        list->processItems();
+        finalizeItemsDelete(par, deleted);
     }
 }
 void PlaylistDock::clear()
@@ -555,11 +638,22 @@ void PlaylistDock::copy()
     }
     QApplication::clipboard()->setMimeData(mimeData);
 }
-void PlaylistDock::paste()
+void PlaylistDock::paste(bool play)
 {
     const QMimeData *mimeData = QApplication::clipboard()->mimeData();
     if (Functions::chkMimeData(mimeData))
-        list->add(Functions::getUrlsFromMimeData(mimeData), list->selectedItems().count() ? list->currentItem() : nullptr);
+    {
+        const auto urls = Functions::getUrlsFromMimeData(mimeData);
+        if (play)
+        {
+            addAndPlay(urls);
+        }
+        else
+        {
+            list->dontUpdateAfterAdd = false;
+            list->add(urls, list->selectedItems().count() ? list->currentItem() : nullptr);
+        }
+    }
 }
 void PlaylistDock::renameGroup()
 {
@@ -571,7 +665,7 @@ void PlaylistDock::renameGroup()
 void PlaylistDock::entryProperties()
 {
     bool sync, accepted;
-    EntryProperties eP(this, list->currentItem(), sync, accepted);
+    EntryProperties eP(this, list, sync, accepted);
     if (accepted)
     {
         if (sync)
