@@ -45,12 +45,19 @@ public:
     void add(const QString &);
     void addAndPlay(const QString &);
 
+    void remove(const QString &url);
+    void remove(const QStringList &urls);
+
     void scrollToCurrectItem();
 
     inline QWidget *findEdit() const
     {
         return (QWidget *)findE;
     }
+
+private:
+    void showEvent(QShowEvent *e) override;
+
 private:
     void expandTree(QTreeWidgetItem *);
 
@@ -60,7 +67,10 @@ private:
 
     void doGroupSync(bool quick, QTreeWidgetItem *tWI, bool quickRecursive = true);
 
-    void deleteTreeWidgetItem(QTreeWidgetItem *tWI);
+    bool maybeDeleteTreeWidgetItem(QTreeWidgetItem *tWI);
+
+    QTreeWidgetItem *initializeItemsDelete();
+    void finalizeItemsDelete(QTreeWidgetItem *par, bool deleted);
 
     QWidget mainW;
     PlaylistWidget *list;
@@ -88,11 +98,13 @@ public slots:
     void clearCurrentPlaying();
     void setCurrentPlaying();
     void newGroup();
-    void delEntries();
+    void delEntries(bool fromDisk);
     void delNonGroupEntries(bool force = false);
     void clear();
     void copy();
-    void paste();
+public:
+    void paste(bool play);
+public slots:
     void renameGroup();
     void entryProperties();
     void timeSort1();
