@@ -95,7 +95,7 @@ public:
     {}
 
     QGroupBox *toAssGB;
-    QCheckBox *colorsAndBordersB, *marginsAndAlignmentB, *fontsB, *overridePlayResB;
+    QCheckBox *colorsAndBordersB, *marginsAndAlignmentB, *fontsB;
 };
 class Page5 : public OSDSettingsW
 {
@@ -132,6 +132,15 @@ void SettingsWidget::InitSettings()
 {
     Settings &QMPSettings = QMPlay2Core.getSettings();
 
+    auto getInitialOutpuitFilePath = [] {
+        QString outputFilePath = Settings("Downloader").getString("DownloadsDirPath");
+        while (outputFilePath.endsWith("/"))
+            outputFilePath.chop(1);
+        if (outputFilePath.isEmpty())
+            outputFilePath = QStandardPaths::standardLocations(QStandardPaths::DownloadLocation).value(0, QDir::homePath());
+        return outputFilePath;
+    };
+
     QTextCodec *codec = QTextCodec::codecForLocale();
     QMPSettings.init("FallbackSubtitlesEncoding", codec ? codec->name().constData() : "System");
 
@@ -141,7 +150,8 @@ void SettingsWidget::InitSettings()
 
     QMPSettings.init("AudioLanguage", QString());
     QMPSettings.init("SubtitlesLanguage", QString());
-    QMPSettings.init("screenshotPth", QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).value(0, QDir::homePath()));
+    QMPSettings.init("screenshotPth", []{return QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).value(0, QDir::homePath());});
+    QMPSettings.init("OutputFilePath", getInitialOutpuitFilePath);
 #ifdef Q_OS_WIN
     QMPSettings.init("screenshotFormat", ".bmp");
 #else
@@ -159,8 +169,7 @@ void SettingsWidget::InitSettings()
     QMPSettings.init("AutoOpenVideoWindow", true);
     QMPSettings.init("AutoRestoreMainWindowOnVideo", true);
 #ifdef UPDATES
-    if (!QMPSettings.contains("AutoUpdates"))
-        QMPSettings.init("AutoUpdates", !QFile::exists(QMPlay2Core.getShareDir() + "noautoupdates"));
+    QMPSettings.init("AutoUpdates", []{return !QFile::exists(QMPlay2Core.getShareDir() + "noautoupdates");});
 #endif
     QMPSettings.init("MainWidget/TabPositionNorth", false);
 #ifdef QMPLAY2_ALLOW_ONLY_ONE_INSTANCE
@@ -182,6 +191,20 @@ void SettingsWidget::InitSettings()
     QMPSettings.init("Proxy/User", QString());
     QMPSettings.init("Proxy/Password", QString());
 
+#ifdef USE_YOUTUBEDL
+    QMPSettings.init("YtDl/CookiesFromBrowserEnabled", false);
+    QMPSettings.init("YtDl/CookiesFromBrowser", QString());
+# if defined(Q_OS_HAIKU)
+    QMPSettings.init("YtDl/CustomPathEnabled", true);
+    QMPSettings.init("YtDl/CustomPath", "/bin/yt-dlp");
+    QMPSettings.init("YtDl/DontAutoUpdate", true);
+# else
+    QMPSettings.init("YtDl/CustomPathEnabled", false);
+    QMPSettings.init("YtDl/CustomPath", QString());
+    QMPSettings.init("YtDl/DontAutoUpdate", false);
+# endif
+#endif
+
     QMPSettings.init("OpenGL/OnWindow", false);
     QMPSettings.init("OpenGL/VSync", true);
     QMPSettings.init("OpenGL/BypassCompositor", false);
@@ -223,13 +246,13 @@ void SettingsWidget::InitSettings()
     QMPSettings.init("ShowBufferedTimeOnSlider", true);
     QMPSettings.init("WheelAction", true);
     QMPSettings.init("WheelSeek", true);
-    QMPSettings.init("LeftMouseTogglePlay", 0);
+    QMPSettings.init("LeftMouseTogglePlay", static_cast<int>(0));
     QMPSettings.init("MiddleMouseToggleFullscreen", false);
-    QMPSettings.init("AccurateSeek", Qt::PartiallyChecked);
+    QMPSettings.init("AccurateSeek", Qt::Checked);
     QMPSettings.init("UnpauseWhenSeeking", false);
     QMPSettings.init("RestoreAVSState", false);
     QMPSettings.init("DisableSubtitlesAtStartup", false);
-    QMPSettings.init("StoreUrlPos", false);
+    QMPSettings.init("StoreUrlPos", true);
     QMPSettings.init("StoreARatioAndZoom", false);
     QMPSettings.init("SavePos", false);
     QMPSettings.init("KeepZoom", false);
@@ -245,11 +268,10 @@ void SettingsWidget::InitSettings()
     QMPSettings.init("ApplyToASS/ColorsAndBorders", true);
     QMPSettings.init("ApplyToASS/MarginsAndAlignment", false);
     QMPSettings.init("ApplyToASS/FontsAndSpacing", false);
-    QMPSettings.init("ApplyToASS/OverridePlayRes", false);
     QMPSettings.init("ApplyToASS/ApplyToASS", false);
     QMPSettings.init("OSD/Enabled", true);
-    OSDSettingsW::init("Subtitles", 20, 0, 15, 15, 15, 7, 1.5, 1.5, QColor(0xFF, 0xA8, 0x58, 0xFF), Qt::black, Qt::black);
-    OSDSettingsW::init("OSD",       32, 0, 0,  0,  0,  4, 1.5, 1.5, QColor(0xAA, 0xFF, 0x55, 0xFF), Qt::black, Qt::black);
+    OSDSettingsW::init("Subtitles", 20, 0, 15, 15, 15, 7, 1.0, 0.5, QColor(0xFF, 0xFF, 0xFF, 0xFF), Qt::black, Qt::black, false);
+    OSDSettingsW::init("OSD",       32, 0, 0,  0,  0,  4, 1.5, 1.5, QColor(0xAA, 0xFF, 0x55, 0xFF), Qt::black, Qt::black, false);
     DeintSettingsW::init();
     applyProxy();
 }
@@ -309,7 +331,7 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         generalSettingsPage->setupUi(generalSettingsWidget);
 
         for (const auto &fmt : QImageWriter::supportedImageFormats())
-            generalSettingsPage->screenshotFormatB->addItem("." + fmt);
+            generalSettingsPage->screenshotFormatB->addItem(QString("." + fmt));
 
         appendColon(generalSettingsPage->langL);
         appendColon(generalSettingsPage->styleL);
@@ -334,8 +356,14 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         }
         generalSettingsPage->langBox->model()->sort(0);
 
-        generalSettingsPage->styleBox->addItems(QStyleFactory::keys());
-        idx = generalSettingsPage->styleBox->findText(QApplication::style()->objectName(), Qt::MatchFixedString);
+        const auto currStyle = QApplication::style()->objectName();
+        auto styles = QStyleFactory::keys();
+#if defined(Q_OS_WIN) && QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        if (QOperatingSystemVersion::current() < QOperatingSystemVersion::Windows11 && currStyle != "windows11")
+            styles.removeOne("windows11");
+#endif
+        generalSettingsPage->styleBox->addItems(styles);
+        idx = generalSettingsPage->styleBox->findText(currStyle, Qt::MatchFixedString);
         if (idx > -1 && idx < generalSettingsPage->styleBox->count())
             generalSettingsPage->styleBox->setCurrentIndex(idx);
         connect(generalSettingsPage->styleBox, SIGNAL(currentIndexChanged(int)), this, SLOT(chStyle()));
@@ -383,6 +411,10 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         generalSettingsPage->screenshotB->setIcon(QMPlay2Core.getIconFromTheme("folder-open"));
         connect(generalSettingsPage->screenshotB, SIGNAL(clicked()), this, SLOT(chooseScreenshotDir()));
 
+        generalSettingsPage->outputFileE->setText(QMPSettings.getString("OutputFilePath"));
+        generalSettingsPage->outputFileB->setIcon(QMPlay2Core.getIconFromTheme("folder-open"));
+        connect(generalSettingsPage->outputFileB, &QAbstractButton::clicked, this, &SettingsWidget::chooseOutputFileDir);
+
         connect(generalSettingsPage->setAppearanceB, SIGNAL(clicked()), this, SLOT(setAppearance()));
         connect(generalSettingsPage->setKeyBindingsB, SIGNAL(clicked()), this, SLOT(setKeyBindings()));
 
@@ -446,15 +478,47 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         const QIcon viewRefresh = QMPlay2Core.getIconFromTheme("view-refresh");
         generalSettingsPage->clearCoversCache->setIcon(viewRefresh);
         connect(generalSettingsPage->clearCoversCache, SIGNAL(clicked()), this, SLOT(clearCoversCache()));
-#ifdef USE_YOUTUBEDL
-        generalSettingsPage->removeYtDlB->setIcon(QMPlay2Core.getIconFromTheme("list-remove"));
-        connect(generalSettingsPage->removeYtDlB, SIGNAL(clicked()), this, SLOT(removeYouTubeDl()));
-#else
-        generalSettingsPage->removeYtDlB->deleteLater();
-        generalSettingsPage->removeYtDlB = nullptr;
-#endif
+
         generalSettingsPage->resetSettingsB->setIcon(viewRefresh);
         connect(generalSettingsPage->resetSettingsB, SIGNAL(clicked()), this, SLOT(resetSettings()));
+
+#ifdef USE_YOUTUBEDL
+        generalSettingsPage->cookiesFromBrowserCB->setChecked(QMPSettings.getBool("YtDl/CookiesFromBrowserEnabled"));
+        generalSettingsPage->cookiesFromBrowserE->setText(QMPSettings.getString("YtDl/CookiesFromBrowser"));
+
+        generalSettingsPage->customYtDlCB->setChecked(QMPSettings.getBool("YtDl/CustomPathEnabled"));
+        generalSettingsPage->customYtDlE->setText(QMPSettings.getString("YtDl/CustomPath"));
+        generalSettingsPage->customYtDlB->setIcon(QMPlay2Core.getIconFromTheme("folder-open"));
+
+        generalSettingsPage->dontUpdateYtDlCB->setChecked(QMPSettings.getBool("YtDl/DontAutoUpdate"));
+
+        connect(generalSettingsPage->cookiesFromBrowserCB, &QCheckBox::toggled, this, [this](bool checked) {
+            generalSettingsPage->cookiesFromBrowserE->setEnabled(checked);
+        });
+        connect(generalSettingsPage->customYtDlCB, &QCheckBox::toggled, this, [this](bool checked) {
+            generalSettingsPage->customYtDlE->setEnabled(checked);
+            generalSettingsPage->customYtDlB->setEnabled(checked);
+            generalSettingsPage->removeYtDlB->setEnabled(!checked);
+        });
+        connect(generalSettingsPage->customYtDlB, &QToolButton::clicked, this, [this] {
+            auto path = QFileDialog::getOpenFileName(this, tr("Choose youtube-dl script or executable"), generalSettingsPage->customYtDlE->text());
+            if (!path.isEmpty())
+                generalSettingsPage->customYtDlE->setText(path);
+        });
+
+        emit generalSettingsPage->cookiesFromBrowserCB->toggled(generalSettingsPage->cookiesFromBrowserCB->isChecked());
+        emit generalSettingsPage->customYtDlCB->toggled(generalSettingsPage->customYtDlCB->isChecked());
+
+        connect(generalSettingsPage->customYtDlCB, &QCheckBox::toggled, this, [this](bool checked) {
+            generalSettingsPage->dontUpdateYtDlCB->setChecked(checked);
+        });
+
+        generalSettingsPage->removeYtDlB->setIcon(QMPlay2Core.getIconFromTheme("list-remove"));
+        connect(generalSettingsPage->removeYtDlB, &QPushButton::clicked, this, &SettingsWidget::removeYouTubeDl);
+#else
+        generalSettingsPage->ytDlGB->deleteLater();
+        generalSettingsPage->ytDlGB = nullptr;
+#endif
     }
 
     {
@@ -483,6 +547,19 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
     createRendererSettings();
 
     {
+        auto getDesiredVideoHeightIndex = [&] {
+            const int desiredVideoHeight = QMPSettings.getInt("DesiredVideoHeight");
+            if (desiredVideoHeight <= 0)
+                return 0;
+            if (desiredVideoHeight < 720)
+                return 1;
+            if (desiredVideoHeight < 1080)
+                return 2;
+            if (desiredVideoHeight < 2160)
+                return 3;
+            return 4;
+        };
+
         QWidget *playbackSettingsWidget = new QWidget;
         playbackSettingsPage = new Ui::PlaybackSettings;
         playbackSettingsPage->setupUi(playbackSettingsWidget);
@@ -513,6 +590,7 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         playbackSettingsPage->bufferNetworkB->setValue(QMPSettings.getDouble("AVBufferTimeNetwork"));
         playbackSettingsPage->backwardBufferNetworkB->setCurrentIndex(QMPSettings.getUInt("BackwardBuffer"));
         playbackSettingsPage->bufferLiveB->setValue(QMPSettings.getDouble("AVBufferTimeNetworkLive"));
+        playbackSettingsPage->desiredVideoHeightB->setCurrentIndex(getDesiredVideoHeightIndex());
         playbackSettingsPage->playIfBufferedB->setValue(QMPSettings.getDouble("PlayIfBuffered"));
         playbackSettingsPage->maxVolB->setValue(QMPSettings.getInt("MaxVol"));
 
@@ -657,9 +735,6 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         page4->fontsB = new QCheckBox(tr("Fonts and spacing"));
         page4->fontsB->setChecked(QMPSettings.getBool("ApplyToASS/FontsAndSpacing"));
 
-        page4->overridePlayResB = new QCheckBox(tr("Use the same size"));
-        page4->overridePlayResB->setChecked(QMPSettings.getBool("ApplyToASS/OverridePlayRes"));
-
         page4->toAssGB = new QGroupBox(tr("Apply for ASS/SSA subtitles"));
         page4->toAssGB->setCheckable(true);
         page4->toAssGB->setChecked(QMPSettings.getBool("ApplyToASS/ApplyToASS"));
@@ -668,7 +743,6 @@ SettingsWidget::SettingsWidget(int page, const QString &moduleName, QWidget *vid
         page4ToAssLayout->addWidget(page4->colorsAndBordersB, 0, 0, 1, 1);
         page4ToAssLayout->addWidget(page4->marginsAndAlignmentB, 1, 0, 1, 1);
         page4ToAssLayout->addWidget(page4->fontsB, 0, 1, 1, 1);
-        page4ToAssLayout->addWidget(page4->overridePlayResB, 1, 1, 1, 1);
 
         page4->addWidget(page4->toAssGB);
     }
@@ -1124,6 +1198,25 @@ void SettingsWidget::restartApp()
     QMPlay2GUI.mainW->close();
 }
 
+QString SettingsWidget::chooseDir(const QString &currPth)
+{
+    const QString dir = QFileDialog::getExistingDirectory(this, tr("Choose directory"), currPth);
+    if (!dir.isEmpty())
+    {
+        const QFileInfo dirInfo(dir);
+#if !defined(Q_OS_WIN) && !defined(Q_OS_ANDROID)
+        if (dirInfo.isDir() && dirInfo.isWritable())
+#else
+        if (dirInfo.isDir())
+#endif
+        {
+            return dir;
+        }
+        QMessageBox::warning(this, QString(), tr("Cannot change the directory"));
+    }
+    return QString();
+}
+
 inline QString SettingsWidget::getSelectedProfile()
 {
     return !generalSettingsPage->profileB->currentIndex() ? "/" : generalSettingsPage->profileB->currentText();
@@ -1187,6 +1280,7 @@ void SettingsWidget::apply()
             QMPSettings.set("SubtitlesLanguage", generalSettingsPage->subsLangB->currentIndex() > 0 ? generalSettingsPage->subsLangB->currentText() : QString());
             QMPSettings.set("screenshotPth", generalSettingsPage->screenshotE->text());
             QMPSettings.set("screenshotFormat", generalSettingsPage->screenshotFormatB->currentText());
+            QMPSettings.set("OutputFilePath", generalSettingsPage->outputFileE->text());
             QMPSettings.set("ShowCovers", generalSettingsPage->showCoversGB->isChecked());
             QMPSettings.set("BlurCovers", generalSettingsPage->blurCoversB->isChecked());
             QMPSettings.set("ShowDirCovers", generalSettingsPage->showDirCoversB->isChecked());
@@ -1219,6 +1313,14 @@ void SettingsWidget::apply()
             qobject_cast<QMainWindow *>(QMPlay2GUI.mainW)->setTabPosition(Qt::AllDockWidgetAreas, generalSettingsPage->tabsNorths->isChecked() ? QTabWidget::North : QTabWidget::South);
             applyProxy();
 
+#ifdef USE_YOUTUBEDL
+            QMPSettings.set("YtDl/CookiesFromBrowserEnabled", generalSettingsPage->cookiesFromBrowserCB->isChecked() && !generalSettingsPage->cookiesFromBrowserE->text().simplified().isEmpty());
+            QMPSettings.set("YtDl/CookiesFromBrowser", generalSettingsPage->cookiesFromBrowserE->text());
+            QMPSettings.set("YtDl/CustomPathEnabled", generalSettingsPage->customYtDlCB->isChecked() && !generalSettingsPage->customYtDlE->text().trimmed().isEmpty());
+            QMPSettings.set("YtDl/CustomPath", generalSettingsPage->customYtDlE->text());
+            QMPSettings.set("YtDl/DontAutoUpdate", generalSettingsPage->dontUpdateYtDlCB->isChecked());
+#endif
+
             if (generalSettingsPage->trayNotifiesDefault)
                 Notifies::setNativeFirst(!generalSettingsPage->trayNotifiesDefault->isChecked());
 
@@ -1247,6 +1349,22 @@ void SettingsWidget::apply()
         }
         case 2:
         {
+            auto getDesiredVideoHeight = [this] {
+                switch (playbackSettingsPage->desiredVideoHeightB->currentIndex())
+                {
+                    case 1:
+                        return 480;
+                    case 2:
+                        return 720;
+                    case 3:
+                        return 1080;
+                    case 4:
+                        return 2160;
+                    default:
+                        return 0;
+                }
+            };
+
             QMPSettings.set("ShortSeek", playbackSettingsPage->shortSeekB->value());
             QMPSettings.set("LongSeek", playbackSettingsPage->longSeekB->value());
             QMPSettings.set("AVBufferLocal", playbackSettingsPage->bufferLocalB->value());
@@ -1254,6 +1372,7 @@ void SettingsWidget::apply()
             QMPSettings.set("BackwardBuffer", playbackSettingsPage->backwardBufferNetworkB->currentIndex());
             QMPSettings.set("AVBufferTimeNetworkLive", playbackSettingsPage->bufferLiveB->value());
             QMPSettings.set("PlayIfBuffered", playbackSettingsPage->playIfBufferedB->value());
+            QMPSettings.set("DesiredVideoHeight", getDesiredVideoHeight());
             QMPSettings.set("ForceSamplerate", playbackSettingsPage->forceSamplerate->isChecked());
             QMPSettings.set("Samplerate", playbackSettingsPage->samplerateB->value());
             QMPSettings.set("MaxVol", playbackSettingsPage->maxVolB->value());
@@ -1317,7 +1436,6 @@ void SettingsWidget::apply()
             QMPSettings.set("ApplyToASS/ColorsAndBorders", page4->colorsAndBordersB->isChecked());
             QMPSettings.set("ApplyToASS/MarginsAndAlignment", page4->marginsAndAlignmentB->isChecked());
             QMPSettings.set("ApplyToASS/FontsAndSpacing", page4->fontsB->isChecked());
-            QMPSettings.set("ApplyToASS/OverridePlayRes", page4->overridePlayResB->isChecked());
             QMPSettings.set("ApplyToASS/ApplyToASS", page4->toAssGB->isChecked());
             break;
         case 5:
@@ -1454,10 +1572,16 @@ void SettingsWidget::moveModule()
 }
 void SettingsWidget::chooseScreenshotDir()
 {
-    const QString dir = QFileDialog::getExistingDirectory(this, tr("Choose directory"), generalSettingsPage->screenshotE->text());
+    const QString dir = chooseDir(generalSettingsPage->screenshotE->text());
     if (!dir.isEmpty())
         generalSettingsPage->screenshotE->setText(dir);
 }
+void SettingsWidget::chooseOutputFileDir()
+{
+    const QString dir = chooseDir(generalSettingsPage->outputFileE->text());
+    if (!dir.isEmpty())
+        generalSettingsPage->outputFileE->setText(dir);
+}
 void SettingsWidget::setAppearance()
 {
     Appearance(this).exec();
