@@ -19,11 +19,14 @@
 #include <Downloader.hpp>
 
 #include <Functions.hpp>
-#include <MkvMuxer.hpp>
+#include <StreamMuxer.hpp>
 #include <Demuxer.hpp>
 #include <Packet.hpp>
 #include <Reader.hpp>
 
+#include <QSet>
+#include <QDir>
+#include <QHash>
 #include <QMenu>
 #include <QTimer>
 #include <QLabel>
@@ -32,8 +35,8 @@
 #include <QWindow>
 #include <QProcess>
 #include <QMimeData>
+#include <QFileInfo>
 #include <QClipboard>
-#include <QFileDialog>
 #include <QTreeWidget>
 #include <QToolButton>
 #include <QGridLayout>
@@ -45,7 +48,6 @@
 #include <QProgressBar>
 #include <QApplication>
 #include <QElapsedTimer>
-#include <QStandardPaths>
 #include <QLoggingCategory>
 #include <QDialogButtonBox>
 
@@ -451,16 +453,12 @@ void DownloadItemW::startConversion()
     maybeAddAbsolutePath(convertCommand);
 
     qDebug() << "Starting conversion:" << convertCommand.toUtf8().constData();
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
     auto args = QProcess::splitCommand(convertCommand);
     if (!args.isEmpty())
     {
         const QString program = args.takeFirst();
         m_convertProcess->start(program, args);
     }
-#else
-    m_convertProcess->start(convertCommand);
-#endif
 }
 void DownloadItemW::deleteConvertProcess()
 {
@@ -605,10 +603,12 @@ void DownloaderThread::run()
             }
 
     const auto getFilePath = [&]()->QString {
+        const auto downloadsDirPath = Functions::cleanPath(QMPlay2Core.getSettings().getString("OutputFilePath"));
         QString filePath;
         quint16 num = 0;
+        Q_ASSERT(downloadsDirPath.endsWith("/"));
         do
-            filePath = downloadLW->getDownloadsDirPath() + (num ? (QString::number(num) + "_") : QString()) + Functions::cleanFileName(name);
+            filePath = downloadsDirPath + (num ? (QString::number(num) + "_") : QString()) + Functions::cleanFileName(name);
         while (QFile::exists(filePath) && ++num < 0xFFFF);
         if (num == 0xFFFF)
             filePath.clear();
@@ -628,13 +628,66 @@ void DownloaderThread::run()
     bool err = true;
 
     const bool isFFmpeg = newUrl.startsWith("FFmpeg://");
-    const bool isHLS = (!isFFmpeg && (newUrl.endsWith(".m3u8", Qt::CaseInsensitive) || newUrl.contains(".m3u8?", Qt::CaseInsensitive)));
-    if (isFFmpeg || isHLS)
+    const bool isLive = !isFFmpeg && (newUrl.endsWith(".m3u8", Qt::CaseInsensitive)
+                                   || newUrl.endsWith(".mpd", Qt::CaseInsensitive)
+                                   || newUrl.contains(".m3u8?", Qt::CaseInsensitive));
+    if (isFFmpeg || isLive || !prefix.isEmpty())
     {
         IOController<Demuxer> &demuxer = ioCtrl.toRef<Demuxer>();
         QMPlay2Core.setWorking(true);
         if (Demuxer::create(newUrl, demuxer) && !demuxer.isAborted())
         {
+            QList<StreamInfo *> streamsInfo = demuxer->streamsInfo();
+            QHash<int, int> selectedStreamsMap;
+
+            // Download only first default video stream
+            {
+                QSet<int> selectedStreams;
+                bool hasDefaultVideoStream = false;
+                bool videoStreamAdded = false;
+                int i = 0;
+                selectedStreams.reserve(streamsInfo.count());
+                selectedStreamsMap.reserve(streamsInfo.count());
+                for (auto &&streamInfo : std::as_const(streamsInfo))
+                {
+                    if (streamInfo->params->codec_type == AVMEDIA_TYPE_VIDEO && streamInfo->is_default)
+                    {
+                        hasDefaultVideoStream = true;
+                        break;
+                    }
+                }
+                for (auto it = streamsInfo.begin(); it != streamsInfo.end();)
+                {
+                    const auto streamInfo = *it;
+                    auto insertStream = [&] {
+                        selectedStreamsMap[i] = selectedStreams.size();
+                        selectedStreams.insert(i);
+                    };
+                    if (streamInfo->params->codec_type == AVMEDIA_TYPE_VIDEO)
+                    {
+                        if (!videoStreamAdded && (!hasDefaultVideoStream || streamInfo->is_default))
+                        {
+                            insertStream();
+                            videoStreamAdded = true;
+                            ++it;
+                        }
+                        else
+                        {
+                            it = streamsInfo.erase(it);
+                        }
+                    }
+                    else
+                    {
+                        insertStream();
+                        ++it;
+                    }
+                    ++i;
+                }
+                demuxer->selectStreams(selectedStreams);
+            }
+
+            const auto [ext, fmt] = Functions::determineExtFmt(streamsInfo);
+
             if (name.isEmpty())
                 name = demuxer->title();
             if (name.isEmpty())
@@ -644,15 +697,14 @@ void DownloaderThread::run()
                 if (idx > -1)
                     name.remove(idx, name.size() - idx);
             }
-            if (!name.endsWith(".mkv", Qt::CaseInsensitive))
-                name += ".mkv";
+            if (!name.endsWith("." + ext, Qt::CaseInsensitive))
+                name += "." + ext;
             emit listSig(NAME);
 
             QString filePath = getFilePath();
             if (!filePath.isEmpty())
             {
-                const QList<StreamInfo *> streamsInfo = demuxer->streamsInfo();
-                MkvMuxer muxer(filePath, streamsInfo);
+                StreamMuxer muxer(filePath, streamsInfo, fmt);
                 if (muxer.isOk())
                 {
                     const double length = demuxer->length();
@@ -661,7 +713,7 @@ void DownloaderThread::run()
                     double bytePos = 0.0;
                     double pos = 0.0;
 
-                    emit listSig(SET, (size < 0 || isHLS) ? (length < 0 ? -1 : -2) : size, filePath);
+                    emit listSig(SET, (size < 0 || isLive) ? (length < 0 ? -1 : -2) : size, filePath);
                     err = false;
                     speedT.start();
                     while (!demuxer.isAborted())
@@ -670,6 +722,7 @@ void DownloaderThread::run()
                         int idx = -1;
                         if (!demuxer->read(packet, idx))
                             break;
+                        idx = selectedStreamsMap.value(idx, -1);
                         if (idx < 0 || idx >= streamsInfo.count())
                             continue;
 
@@ -862,13 +915,6 @@ void Downloader::init()
     connect(m_convertsMenu->addAction(tr("&Add")), &QAction::triggered, this, &Downloader::addConvertPreset);
     m_convertsMenu->addSeparator();
 
-    setDownloadsDirB = new QToolButton;
-    setDownloadsDirB->setIcon(QMPlay2Core.getIconFromTheme("folder-open"));
-    setDownloadsDirB->setText(tr("Download directory"));
-    setDownloadsDirB->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-    setDownloadsDirB->setToolTip(tr("Choose directory for downloaded files"));
-    connect(setDownloadsDirB, SIGNAL(clicked()), this, SLOT(setDownloadsDir()));
-
     clearFinishedB = new QToolButton;
     clearFinishedB->setIcon(QMPlay2Core.getIconFromTheme("archive-remove"));
     clearFinishedB->setToolTip(tr("Clear completed downloads"));
@@ -879,44 +925,26 @@ void Downloader::init()
     addUrlB->setToolTip(tr("Enter the address for download"));
     connect(addUrlB, SIGNAL(clicked()), this, SLOT(addUrl()));
 
+#ifndef Q_OS_ANDROID
     m_convertsPresetsB = new QToolButton;
     m_convertsPresetsB->setIcon(QMPlay2Core.getIconFromTheme("list-add"));
     m_convertsPresetsB->setToolTip(tr("Add, modify, or remove conversion presets"));
     m_convertsPresetsB->setPopupMode(QToolButton::InstantPopup);
     m_convertsPresetsB->setMenu(m_convertsMenu);
+#else
+    m_convertsPresetsB = nullptr;
+#endif
 
     layout = new QGridLayout(this);
     layout->setContentsMargins(0, 0, 0, 0);
     layout->addWidget(downloadLW, 0, 0, 1, 6);
-    layout->addWidget(setDownloadsDirB, 1, 0, 1, 1);
     layout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum), 1, 1, 1, 1);
     layout->addWidget(clearFinishedB, 1, 2, 1, 1);
     layout->addWidget(addUrlB, 1, 3, 1, 1);
+#ifndef Q_OS_ANDROID
     layout->addItem(new QSpacerItem(10, 0, QSizePolicy::Fixed, QSizePolicy::Minimum), 1, 4, 1, 1);
     layout->addWidget(m_convertsPresetsB, 1, 5, 1, 1);
-
-    QString defDownloadPath = QStandardPaths::standardLocations(QStandardPaths::DownloadLocation).value(0, QDir::homePath()) + "/";
-#ifdef Q_OS_WIN
-    defDownloadPath.replace('\\', '/');
 #endif
-    downloadLW->downloadsDirPath = Functions::cleanPath(m_sets.getString("DownloadsDirPath"));
-    if (downloadLW->downloadsDirPath == "/")
-    {
-        downloadLW->downloadsDirPath = defDownloadPath;
-    }
-    else if (downloadLW->downloadsDirPath != defDownloadPath)
-    {
-        const QFileInfo dir(downloadLW->downloadsDirPath);
-#ifndef Q_OS_WIN
-        if (!dir.isDir() || !dir.isWritable())
-#else
-        if (!dir.isDir())
-#endif
-        {
-            downloadLW->downloadsDirPath = defDownloadPath;
-            m_sets.set("DownloadsDirPath", defDownloadPath);
-        }
-    }
 
     // Compatibility
     {
@@ -943,6 +971,7 @@ void Downloader::init()
     }
     // Presets
     {
+#ifndef Q_OS_ANDROID
         const auto createPreset = [this](const QString &name, const QString &data) {
             QAction *act = m_convertsMenu->addAction(name);
             act->setData(data);
@@ -972,6 +1001,7 @@ void Downloader::init()
             createPreset("MP3 224k", g_defaultMp3ConvertCommand);
             createPreset("OGG container", "ffmpeg -i <input/> -vn -sn -c:a copy -f ogg -y <output>%f.ogg</output>");
         }
+#endif
     }
 }
 
@@ -1121,21 +1151,6 @@ bool Downloader::modifyConvertAction(QAction *action, bool addRemoveButton)
     return false;
 }
 
-void Downloader::setDownloadsDir()
-{
-    QFileInfo dir(QFileDialog::getExistingDirectory(this, tr("Choose directory for downloaded files"), downloadLW->downloadsDirPath));
-#ifndef Q_OS_WIN
-    if (dir.isDir() && dir.isWritable())
-#else
-    if (dir.isDir())
-#endif
-    {
-        downloadLW->downloadsDirPath = Functions::cleanPath(dir.filePath());
-        m_sets.set("DownloadsDirPath", downloadLW->downloadsDirPath);
-    }
-    else if (dir.filePath() != QString())
-        QMessageBox::warning(this, DownloaderName, tr("Cannot change downloading files directory"));
-}
 void Downloader::clearFinished()
 {
     const QList<QTreeWidgetItem *> items = downloadLW->findItems(QString(), Qt::MatchContains);
