@@ -54,8 +54,8 @@ private:
 
     NetworkAccess m_net;
 
-    QString m_realTitle, m_realArtist;
     QString m_title, m_artist, m_name;
+    bool m_titleArtistFromDash = false;
 
     QPointer<NetworkReply> m_tekstowoSearchReply;
     QPointer<NetworkReply> m_tekstowoLyricsReply;
