@@ -35,6 +35,14 @@ bool DiscardDeint::filter(QQueue<Frame> &framesQueue)
         Frame videoFrame = m_internalQueue.dequeue();
         const bool TFF = isTopFieldFirst(videoFrame);
         videoFrame.setNoInterlaced();
+#ifdef USE_VULKAN
+        if (videoFrame.vulkanImage())
+        {
+            auto newFrame = getNewFrame(videoFrame);
+            videoFrame.copyData(newFrame.dataArr(), newFrame.linesize());
+            videoFrame = std::move(newFrame);
+        }
+#endif
         for (int p = 0; p < 3; ++p)
         {
             const int linesize = videoFrame.linesize(p);
