@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2024  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -36,6 +36,7 @@ private:
 #include <QCoreApplication>
 
 class QCheckBox;
+class QComboBox;
 class QSpinBox;
 
 class ModuleSettingsWidget final : public Module::SettingsWidget
@@ -47,5 +48,6 @@ private:
     void saveSettings() override;
 
     QSpinBox *refTimeB = nullptr, *sndLenB, *fftSizeB;
+    QComboBox *m_fftLimitFreqB;
     QCheckBox *m_fftLinearScaleB;
 };
