@@ -144,14 +144,9 @@ constexpr auto s_queueFlags = vk::QueueFlagBits::eGraphics | vk::QueueFlagBits::
 vector<uint32_t> Instance::readShader(const QString &fileName)
 {
     const QResource res(":/vulkan/" + fileName + ".spv");
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
     const auto data = res.uncompressedData();
     const auto u32Data = reinterpret_cast<const uint32_t *>(data.data());
     return vector<uint32_t>(u32Data, u32Data + data.size() / sizeof(uint32_t));
-#else
-    const auto resData = reinterpret_cast<const uint32_t *>(res.data());
-    return vector<uint32_t>(resData, resData + res.size() / sizeof(uint32_t));
-#endif
 }
 
 vk::Format Instance::fromFFmpegPixelFormat(int avPixFmt)
@@ -262,7 +257,7 @@ bool Instance::checkFiltersSupported(const shared_ptr<PhysicalDevice> &physicalD
     if (!physicalDevice)
         return false;
 
-    if (!physicalDevice->isGpu() || !physicalDevice->getFeatures().shaderStorageImageWriteWithoutFormat)
+    if (!physicalDevice->isGpu() || !physicalDevice->getFeatures(physicalDevice->dld()).shaderStorageImageWriteWithoutFormat)
         return false;
 
     return hasStorageImage(physicalDevice, vk::Format::eR8Unorm) && hasStorageImage(physicalDevice, vk::Format::eR8G8Unorm);
@@ -284,7 +279,7 @@ Instance::~Instance()
     delete m_qVulkanInstance;
     m_debugUtilsMessanger.reset();
     if (*this)
-        destroy();
+        destroy(nullptr, dld());
 }
 
 void Instance::prepareDestroy()
@@ -295,13 +290,41 @@ void Instance::prepareDestroy()
 
 void Instance::init(bool doObtainPhysicalDevice)
 {
-    static const auto getInstanceProcAddr = AbstractInstance::loadVulkanLibrary(
-        qEnvironmentVariable("QT_VULKAN_LIB").toStdString()
-    );
-    if (!getInstanceProcAddr)
-        throw vk::InitializationFailedError("Can't resolve \"vkGetInstanceProcAddr\" Vulkan function");
+#ifdef Q_OS_UNIX
+    static bool envVarInitialized = [] {
+        if (qEnvironmentVariableIsSet("QT_VULKAN_LIB"))
+            return true;
 
-    AbstractInstance::initDispatchLoaderDynamic(getInstanceProcAddr);
+        QLibrary lib;
+
+        lib.setFileName("libvulkan.so");
+        if (lib.load())
+        {
+            qputenv("QT_VULKAN_LIB", "libvulkan.so");
+            return true;
+        }
+
+        lib.setFileName("libvulkan.so.1");
+        if (lib.load())
+        {
+            qputenv("QT_VULKAN_LIB", "libvulkan.so.1");
+            return true;
+        }
+
+        return true;
+    }();
+    Q_UNUSED(envVarInitialized);
+#endif
+
+    static shared_ptr<vk::DynamicLoader> dl;
+    const auto getInstanceProcAddr = dl
+        ? setVulkanLibrary(dl)
+        : loadVulkanLibrary(qEnvironmentVariable("QT_VULKAN_LIB").toStdString())
+    ;
+    if (!dl)
+        dl = getDl();
+
+    initDispatchLoaderDynamic(getInstanceProcAddr);
     fetchAllExtensions();
 
     vector<const char *> layers;
@@ -332,7 +355,7 @@ void Instance::init(bool doObtainPhysicalDevice)
 
     vk::ApplicationInfo appInfo;
     appInfo.pApplicationName = "QMPlay2";
-    appInfo.apiVersion = AbstractInstance::isVk10()
+    appInfo.apiVersion = isVk10()
         ? VK_API_VERSION_1_0
         : VK_API_VERSION_1_1
     ;
@@ -346,8 +369,8 @@ void Instance::init(bool doObtainPhysicalDevice)
     createInfo.enabledExtensionCount = instanceExtensions.size();
     createInfo.ppEnabledExtensionNames = instanceExtensions.data();
 
-    static_cast<vk::Instance &>(*this) = vk::createInstance(createInfo);
-    AbstractInstance::initDispatchLoaderDynamic(getInstanceProcAddr, *this);
+    static_cast<vk::Instance &>(*this) = vk::createInstance(createInfo, nullptr, dld());
+    initDispatchLoaderDynamic(getInstanceProcAddr, *this);
 
     m_extensions.clear();
     m_extensions.reserve(instanceExtensions.size());
@@ -368,7 +391,7 @@ void Instance::init(bool doObtainPhysicalDevice)
             vk::DebugUtilsMessageTypeFlagBitsEXT::ePerformance
         ;
         debugUtilsMessagngerCreateInfo.pfnUserCallback = reinterpret_cast<PFN_vkDebugUtilsMessengerCallbackEXT>(debugCallback);
-        m_debugUtilsMessanger = createDebugUtilsMessengerEXTUnique(debugUtilsMessagngerCreateInfo);
+        m_debugUtilsMessanger = createDebugUtilsMessengerEXTUnique(debugUtilsMessagngerCreateInfo, nullptr, dld());
     }
     else
     {
@@ -462,7 +485,7 @@ shared_ptr<Device> Instance::createDevice(const shared_ptr<PhysicalDevice> &phys
     physicalDeviceExtensions.push_back(VK_KHR_SAMPLER_YCBCR_CONVERSION_EXTENSION_NAME);
     physicalDeviceExtensions.push_back(VK_KHR_BIND_MEMORY_2_EXTENSION_NAME);
 
-    if (!AbstractInstance::isVk10() && !physicalDevice->isVk10())
+    if (!isVk10() && !physicalDevice->isVk10())
     {
         // Required for Vulkan video decoding
         physicalDeviceExtensions.push_back(VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME);
@@ -493,7 +516,7 @@ shared_ptr<Device> Instance::createDevice(const shared_ptr<PhysicalDevice> &phys
     m_sycf.samplerYcbcrConversion = false;
     m_tsf.timelineSemaphore = false;
     m_s2f.synchronization2 = false;
-    if (!AbstractInstance::isVk10() || checkExtension(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME))
+    if (!isVk10() || checkExtension(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME))
     {
         vk::PhysicalDeviceFeatures2 features2;
         tie(features2, m_sycf, m_tsf, m_s2f) = physicalDevice->getFeatures2KHR<
@@ -501,7 +524,7 @@ shared_ptr<Device> Instance::createDevice(const shared_ptr<PhysicalDevice> &phys
             vk::PhysicalDeviceSamplerYcbcrConversionFeatures,
             vk::PhysicalDeviceTimelineSemaphoreFeatures,
             vk::PhysicalDeviceSynchronization2Features
-        >().get<
+        >(dld()).get<
             vk::PhysicalDeviceFeatures2,
             vk::PhysicalDeviceSamplerYcbcrConversionFeatures,
             vk::PhysicalDeviceTimelineSemaphoreFeatures,
@@ -511,7 +534,7 @@ shared_ptr<Device> Instance::createDevice(const shared_ptr<PhysicalDevice> &phys
     }
     else
     {
-        m_enabledDeviceFeatures.features.shaderStorageImageWriteWithoutFormat = physicalDevice->getFeatures().shaderStorageImageWriteWithoutFormat;
+        m_enabledDeviceFeatures.features.shaderStorageImageWriteWithoutFormat = physicalDevice->getFeatures(dld()).shaderStorageImageWriteWithoutFormat;
     }
     m_enabledDeviceFeatures.pNext = &m_sycf;
     m_sycf.pNext = &m_tsf;
@@ -640,7 +663,7 @@ bool Instance::isCompatibleDevice(const shared_ptr<PhysicalDevice> &physicalDevi
     QString errorString = "Vulkan :: Discarding \"";
     errorString += properties.deviceName.data();
     errorString += "\", because:";
-    for (auto &&error : qAsConst(errors))
+    for (auto &&error : std::as_const(errors))
         errorString += "\n   - " + error;
     qDebug().noquote() << errorString;
 
