@@ -84,6 +84,7 @@ struct VideoPipelineSpecializationData
     vk::Bool32 useBrightnessContrast;
     vk::Bool32 useHueSaturation;
     vk::Bool32 useSharpness;
+    vk::Bool32 negative;
     int trc;
 };
 
@@ -126,6 +127,7 @@ static inline bool isTrcSupported(AVColorTransferCharacteristic trc)
     {
         case AVCOL_TRC_BT709:
         case AVCOL_TRC_SMPTE2084:
+        case AVCOL_TRC_ARIB_STD_B67:
             return true;
         default:
             break;
@@ -281,6 +283,7 @@ void Window::setParams(
     float hue,
     float saturation,
     float sharpness,
+    bool negative,
     AVColorPrimaries colorPrimaries,
     AVColorTransferCharacteristic colorTrc)
 {
@@ -299,6 +302,8 @@ void Window::setParams(
     m_flip = flip;
     m_rotate90 = rotate90;
 
+    m_negative = negative;
+
     if (
         !qFuzzyCompare(m_brightness, brightness) ||
         !qFuzzyCompare(m_contrast, contrast) ||
@@ -318,6 +323,7 @@ void Window::setParams(
     specializationData->useBrightnessContrast = (!qFuzzyIsNull(brightness) || !qFuzzyCompare(contrast, 1.0f));
     specializationData->useHueSaturation = (!qFuzzyIsNull(hue) || !qFuzzyCompare(saturation, 1.0f));
     specializationData->useSharpness = !qFuzzyIsNull(sharpness);
+    specializationData->negative = m_negative;
 
     if (setSphericalView(sphericalView) || (!m_sphericalView && flipRotateChanged))
         resetVerticesBuffer();
@@ -581,7 +587,7 @@ void Window::render()
             if (!osdLockers.empty())
                 renderOSD();
 
-            m.commandBuffer->endRenderPass();
+            m.commandBuffer->endRenderPass(m.commandBuffer->dld());
 
             if (m.videoPipeline && m_instance->hasFiltersOnOtherQueueFamiliy())
                 m.videoPipeline->finalizeObjects(m.commandBuffer, false, true);
@@ -634,7 +640,7 @@ vector<unique_lock<mutex>> Window::prepareOSD(bool &changed)
     vector<unique_lock<mutex>> lockers;
     lockers.reserve(m_osd.size());
 
-    for (auto &&osd : qAsConst(m_osd))
+    for (auto &&osd : std::as_const(m_osd))
     {
         lockers.push_back(osd->lock());
 
@@ -644,6 +650,9 @@ vector<unique_lock<mutex>> Window::prepareOSD(bool &changed)
         m_osdIDs.insert(id);
     }
 
+    if (!changed && m_osdIDs.size() != osdIDs.size())
+        changed = true;
+
     return lockers;
 }
 
@@ -661,7 +670,7 @@ void Window::beginRenderPass(uint32_t imageIdx)
         renderPassBeginInfo.pClearValues = &clearValue;
     }
 
-    m.commandBuffer->beginRenderPass(renderPassBeginInfo, vk::SubpassContents::eInline);
+    m.commandBuffer->beginRenderPass(renderPassBeginInfo, vk::SubpassContents::eInline, m.commandBuffer->dld());
 }
 
 void Window::maybeClear(uint32_t imageIdx)
@@ -675,7 +684,7 @@ void Window::maybeClear(uint32_t imageIdx)
 
     vk::ClearRect clearRect(vk::Rect2D(vk::Offset2D(), m.swapChain->size()), 0, 1);
 
-    m.commandBuffer->clearAttachments(clearAttachment, clearRect);
+    m.commandBuffer->clearAttachments(clearAttachment, clearRect, m.commandBuffer->dld());
 
     m.clearedImages.insert(imageIdx);
 }
@@ -694,9 +703,9 @@ void Window::renderOSD()
     vector<MemoryObjectDescrs> osdAvMemoryObjects;
     vector<MemoryObjectDescrs> osdAssMemoryObjects;
 
-    vector<const QMPlay2OSD::Image *> osdImages;
+    vector<pair<const QMPlay2OSD::Image *, const QMPlay2OSD *>> osdImages;
 
-    for (auto &&osd : qAsConst(m_osd))
+    for (auto &&osd : std::as_const(m_osd))
     {
         osd->iterate([&](const QMPlay2OSD::Image &image) {
             if (image.dataBufferView->buffer()->device() != m.device)
@@ -721,7 +730,7 @@ void Window::renderOSD()
                 });
             }
 
-            osdImages.push_back(&image);
+            osdImages.emplace_back(&image, osd.get());
         });
     }
 
@@ -756,7 +765,7 @@ void Window::renderOSD()
 
     size_t osdAvMemoryObjectsIdx = 0;
     size_t osdAssMemoryObjectsIdx = 0;
-    for (auto &&img : osdImages)
+    for (auto &&[img, osd] : osdImages)
     {
         QVector2D multiplier(Qt::Uninitialized);
         shared_ptr<GraphicsPipeline> osdPipeline;
@@ -766,8 +775,8 @@ void Window::renderOSD()
         if (img->paletteBufferView)
         {
             multiplier = QVector2D(
-                static_cast<float>(m_scaledSize.width())  / static_cast<float>(m_imgSize.width()),
-                static_cast<float>(m_scaledSize.height()) / static_cast<float>(m_imgSize.height())
+                static_cast<double>(m_scaledSize.width())  / static_cast<double>(m_imgSize.width()),
+                static_cast<double>(m_scaledSize.height()) / static_cast<double>(m_imgSize.height())
             );
             osdPipeline = m.osdAvPipeline;
             osdDescriptorPool = m.osdAvDescriptorPool;
@@ -781,18 +790,16 @@ void Window::renderOSD()
             memObjectDescrs = &osdAssMemoryObjects[osdAssMemoryObjectsIdx++];
         }
 
+        const auto imgRect = osd->getRect(*img);
+
         QMatrix4x4 mat;
         mat.translate(
-            (-1.0f - m_osdOffset.x())
-                + (m_subsRect.x() * 2.0f / winSize.width())
-                + (img->rect.x() * 2.0f * multiplier.x() / winSize.width()),
-            (-1.0f - m_osdOffset.y())
-                + (m_subsRect.y() * 2.0f / winSize.height())
-                + (img->rect.y() * 2.0f * multiplier.y() / winSize.height())
+            (-1.0 - m_osdOffset.x()) + (m_subsRect.x() + imgRect.x() * multiplier.x()) * 2.0 / winSize.width(),
+            (-1.0 - m_osdOffset.y()) + (m_subsRect.y() + imgRect.y() * multiplier.y()) * 2.0 / winSize.height()
         );
         mat.scale(
-            img->rect.width()  * 2.0f * multiplier.x() / winSize.width(),
-            img->rect.height() * 2.0f * multiplier.y() / winSize.height()
+            imgRect.width()  * 2.0 * multiplier.x() / winSize.width(),
+            imgRect.height() * 2.0 * multiplier.y() / winSize.height()
         );
 
         auto pushConstants = osdPipeline->pushConstants<OSDPushConstants>();
@@ -841,7 +848,7 @@ bool Window::ensureSurfaceAndRenderPass()
     if (!m.surface)
         return false;
 
-    const auto surfaceFormats = m_physicalDevice->getSurfaceFormatsKHR(m.surface);
+    const auto surfaceFormats = m_physicalDevice->getSurfaceFormatsKHR(m.surface, m_physicalDevice->dld());
     if (surfaceFormats.empty())
         return false;
 
@@ -907,7 +914,7 @@ void Window::ensureSwapChain()
         }
         case Qt::PartiallyChecked:
         {
-            const auto availPresentModes = m_physicalDevice->getSurfacePresentModesKHR(m.surface);
+            const auto availPresentModes = m_physicalDevice->getSurfacePresentModesKHR(m.surface, m_physicalDevice->dld());
             if (find(availPresentModes.begin(), availPresentModes.end(), vk::PresentModeKHR::eMailbox) != availPresentModes.end())
             {
                 presentModes.push_back(vk::PresentModeKHR::eMailbox);
@@ -1084,7 +1091,7 @@ void Window::fillVerticesBuffer()
     auto data = verticesBuffer->map<uint8_t>();
 
     auto bindVertexBuffer = [=] {
-        m.commandBuffer->bindVertexBuffers(0, {*m.verticesBuffer, *m.verticesBuffer}, {0, verticesSize});
+        m.commandBuffer->bindVertexBuffers(0, {*m.verticesBuffer, *m.verticesBuffer}, {0, verticesSize}, m.commandBuffer->dld());
     };
 
     if (!m_sphericalView)
@@ -1143,8 +1150,8 @@ void Window::fillVerticesBuffer()
 
         m.commandBufferDrawFn = [=] {
             bindVertexBuffer();
-            m.commandBuffer->bindIndexBuffer(*m.verticesBuffer, verticesSize + texCoordsSize, vk::IndexType::eUint16);
-            m.commandBuffer->drawIndexed(nIndices, 1, 0, 0, 0);
+            m.commandBuffer->bindIndexBuffer(*m.verticesBuffer, verticesSize + texCoordsSize, vk::IndexType::eUint16, m.commandBuffer->dld());
+            m.commandBuffer->drawIndexed(nIndices, 1, 0, 0, 0, m.commandBuffer->dld());
         };
     }
 
@@ -1470,7 +1477,7 @@ void Window::ensureBicubic()
 
 inline void Window::draw4Vertices()
 {
-    m.commandBuffer->draw(4, 1, 0, 0);
+    m.commandBuffer->draw(4, 1, 0, 0, m.commandBuffer->dld());
 }
 
 void Window::resetSwapChainAndGraphicsPipelines(bool takeOldSwapChain) try
@@ -1490,7 +1497,7 @@ void Window::resetSwapChainAndGraphicsPipelines(bool takeOldSwapChain) try
 
     if (!m.queueLocker.owns_lock())
         m.queueLocker = m.queue->lock();
-    m.queue->waitIdle();
+    m.queue->waitIdle(m.queue->dld());
     m.queueLocker.unlock();
 
     m.commandBuffer->resetStoredData();
