@@ -16,7 +16,6 @@ namespace QmVk {
 
 struct CommandBuffer::StoredData
 {
-    unordered_set<MemoryObjectDescrs> memoryObjects;
     unordered_set<shared_ptr<DescriptorSet>> descriptorSets;
     unordered_set<shared_ptr<MemoryObjectBase>> memoryObjectsBase;
 };
@@ -36,6 +35,7 @@ CommandBuffer::CommandBuffer(
     const shared_ptr<Queue> &queue,
     Priv)
     : m_queue(queue)
+    , m_dld(m_queue->dld())
 {}
 CommandBuffer::~CommandBuffer()
 {}
@@ -47,13 +47,13 @@ void CommandBuffer::init()
     vk::CommandPoolCreateInfo commandPoolCreateInfo;
     commandPoolCreateInfo.flags = vk::CommandPoolCreateFlagBits::eTransient | vk::CommandPoolCreateFlagBits::eResetCommandBuffer;
     commandPoolCreateInfo.queueFamilyIndex = m_queue->queueFamilyIndex();
-    m_commandPool = device->createCommandPoolUnique(commandPoolCreateInfo);
+    m_commandPool = device->createCommandPoolUnique(commandPoolCreateInfo, nullptr, dld());
 
     vk::CommandBufferAllocateInfo commandBufferAllocateInfo;
     commandBufferAllocateInfo.commandPool = *m_commandPool;
     commandBufferAllocateInfo.level = vk::CommandBufferLevel::ePrimary;
     commandBufferAllocateInfo.commandBufferCount = 1;
-    static_cast<vk::CommandBuffer &>(*this) = device->allocateCommandBuffers(commandBufferAllocateInfo)[0];
+    static_cast<vk::CommandBuffer &>(*this) = device->allocateCommandBuffers(commandBufferAllocateInfo, dld())[0];
 }
 
 void CommandBuffer::storeData(
@@ -63,8 +63,10 @@ void CommandBuffer::storeData(
     if (!m_storedData)
         m_storedData = make_unique<StoredData>();
 
-    m_storedData->memoryObjects.insert(memoryObjects);
     m_storedData->descriptorSets.insert(descriptorSet);
+    memoryObjects.iterateMemoryObjects([this](const shared_ptr<MemoryObjectBase> &object) {
+        m_storedData->memoryObjectsBase.insert(object);
+    });
 }
 void CommandBuffer::storeData(
     const shared_ptr<MemoryObjectBase> &memoryObjectBase)
@@ -79,7 +81,6 @@ void CommandBuffer::resetStoredData()
     if (!m_storedData)
         return;
 
-    m_storedData->memoryObjects.clear();
     m_storedData->descriptorSets.clear();
     m_storedData->memoryObjectsBase.clear();
 }
@@ -88,10 +89,10 @@ void CommandBuffer::resetAndBegin()
 {
     if (m_resetNeeded)
     {
-        reset(vk::CommandBufferResetFlags());
+        reset(vk::CommandBufferResetFlags(), dld());
         resetStoredData();
     }
-    begin(vk::CommandBufferBeginInfo(vk::CommandBufferUsageFlagBits::eOneTimeSubmit));
+    begin(vk::CommandBufferBeginInfo(vk::CommandBufferUsageFlagBits::eOneTimeSubmit), dld());
     m_resetNeeded = true;
 }
 void CommandBuffer::endSubmitAndWait(
@@ -106,7 +107,7 @@ void CommandBuffer::endSubmitAndWait(
 {
     unique_lock<mutex> queueLock;
 
-    end();
+    end(dld());
 
     if (lock)
         queueLock = m_queue->lock();
