@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: MIT
 /*
    QmVk - simple Vulkan library created for QMPlay2
-   Copyright (C) 2020-2023 Błażej Szczygieł
+   Copyright (C) 2020-2024 Błażej Szczygieł
 */
 
 #pragma once
@@ -10,6 +10,8 @@
 
 #include "MemoryObjectDescr.hpp"
 
+#include <functional>
+
 namespace QmVk {
 
 using namespace std;
@@ -19,6 +21,8 @@ class QMVK_EXPORT MemoryObjectDescrs
     friend class hash<MemoryObjectDescrs>;
     friend class Pipeline;
 
+    using Callback = function<void(const shared_ptr<MemoryObjectBase> &)>;
+
 public:
     MemoryObjectDescrs();
     MemoryObjectDescrs(const initializer_list<MemoryObjectDescr> &memoryObjects);
@@ -28,16 +32,20 @@ public:
 public:
     void append(const MemoryObjectDescr &memoryObjectDescr);
 
-    vector<vk::DescriptorPoolSize> fetchDescriptorTypes() const;
+    vector<DescriptorType> fetchDescriptorTypes() const;
     vector<DescriptorInfo> fetchDescriptorInfos() const;
 
+    void iterateMemoryObjects(const Callback &callback) const;
+
 private:
     void prepareObjects(
         vk::CommandBuffer commandBuffer,
         vk::PipelineStageFlags pipelineStageFlags
     ) const;
     void finalizeObjects(
-        vk::CommandBuffer commandBuffer
+        vk::CommandBuffer commandBuffer,
+        bool genMipmapsOnWrite,
+        bool resetPipelineStageFlags
     ) const;
 
 public:
