@@ -44,6 +44,7 @@ private:
 
 public:
     inline shared_ptr<Device> device() const;
+    inline const vk::DispatchLoaderDynamic &dld() const;
 
     inline uint32_t queueFamilyIndex() const;
     inline uint32_t queueIndex() const;
@@ -55,6 +56,7 @@ public:
 
 private:
     const shared_ptr<Device> m_device;
+    const vk::DispatchLoaderDynamic &m_dld;
     const uint32_t m_queueFamilyIndex;
     const uint32_t m_queueIndex;
 
@@ -70,6 +72,10 @@ shared_ptr<Device> Queue::device() const
 {
     return m_device;
 }
+const vk::DispatchLoaderDynamic &Queue::dld() const
+{
+    return m_dld;
+}
 
 uint32_t Queue::queueFamilyIndex() const
 {
