@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: MIT
 /*
    QmVk - simple Vulkan library created for QMPlay2
-   Copyright (C) 2020-2023 Błażej Szczygieł
+   Copyright (C) 2020-2024 Błażej Szczygieł
 */
 
 #include "ShaderModule.hpp"
@@ -39,7 +39,7 @@ void ShaderModule::init(const vector<uint32_t> &data)
     createInfo.codeSize = data.size() * sizeof(uint32_t);
     createInfo.pCode = data.data();
 
-    m_shaderModule = m_device->createShaderModuleUnique(createInfo);
+    m_shaderModule = m_device->createShaderModuleUnique(createInfo, nullptr, m_device->dld());
 }
 
 vk::PipelineShaderStageCreateInfo ShaderModule::getPipelineShaderStageCreateInfo(
